==> selectorablium-2.0.3.tgz: package/compiled/storage_freak.js <==
var __slice = [].slice;

define(['jquery', 'shims/local_storage_shim'], function($, LocalStorageShim) {
  var StorageFreak;
  StorageFreak = (function() {
    var fuzzy_sort, lexicographical_sort, regexp_sort;

    regexp_sort = function(re, a, b) {
      return (function(re_match_a, re_match_b) {
        if (re_match_a && !re_match_b) {
          return -1;
        }
        if (re_match_b && !re_match_a) {
          return 1;
        }
      })(this.config.match_func(re, a.name), this.config.match_func(re, b.name));
    };

    lexicographical_sort = function(a, b) {
      if (a.name < b.name) {
        return -1;
      }
      if (b.name < a.name) {
        return 1;
      }
    };

    fuzzy_sort = function(query, a, b) {
      var re, regexp_result, _i, _len, _ref;
      this.sort_criteria_res || (this.sort_criteria_res = this._createSortingREs(query));
      _ref = this.sort_criteria_res;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        re = _ref[_i];
        regexp_result = regexp_sort.call(this, re, a, b);
        re.lastIndex = 0;
        if (regexp_result) {
          return regexp_result;
        }
      }
      return lexicographical_sort(a, b);
    };

    StorageFreak.prototype._defaults = {
      namespace: 'selectorablium',
      sort_func: (function(query, a, b) {
        return fuzzy_sort.call(this, query, a, b);
      }),
      match_func: (function(RE, name) {
        var result;
        result = RE.test(name);
        RE.lastIndex = 0;
        return result;
      }),
      search_type: 'infix'
    };

    StorageFreak.prototype._required = ['url', 'name', 'query', 'app_name', 'XHRTimeout', 'maxResultsNum', 'localCacheTimeout', 'minCharsForRemoteSearch', 'list_of_replacable_chars'];

    function StorageFreak(options) {
      var attr, k, v, _i, _len, _ref, _ref1;
      if (this instanceof StorageFreak === false) {
        return new StorageFreak(options);
      }
      this.options = options;
      this.config = $.extend({}, this._defaults, this.options);
      _ref = this.config;
      for (k in _ref) {
        v = _ref[k];
        if ($.type(v) === 'function') {
          this.config[k] = $.proxy(v, this);
        }
      }
      _ref1 = this._required;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        attr = _ref1[_i];
        if (!this.config[attr]) {
          throw new Error("'" + attr + "' option is required");
        }
      }
      this._db_prefix = "" + this.config.namespace + "." + this.config.app_name;
      this._data_key = "" + this.config.name + "_data";
      this._timestamp_key = "" + this.config.name + "_timestamp";
      this._data = {};
      this._events = {};
      this.timeout = null;
      this.XHR_dfd = null;
      this._storage = this._getStorage();
    }

    StorageFreak.prototype.on = function(event_name, callback, context) {
      var _base;
      if (context == null) {
        context = null;
      }
      if ((_base = this._events)[event_name] == null) {
        _base[event_name] = [];
      }
      return this._events[event_name].push({
        callback: callback,
        context: context
      });
    };

    StorageFreak.prototype.init = function() {
      if (this._isInvalidData()) {
        return this._getRemoteData('', {
          event_name: 'dbcreate'
        });
      } else {
        this._data = this._get(this._data_key);
        return new $.Deferred().resolve();
      }
    };

    StorageFreak.prototype.add = function(value, text) {
      var new_data;
      if (this._data[text]) {
        return;
      }
      new_data = {};
      new_data[text] = value;
      this._data = $.extend({}, this._data, new_data);
      return this._updateDB(this._data);
    };

    StorageFreak.prototype.searchByValue = function(val) {
      var text, value, _ref;
      _ref = this._data;
      for (text in _ref) {
        value = _ref[text];
        if (value === val) {
          return text;
        }
      }
      return false;
    };


    /**
     * The main API method.
     *
     * It queries the dataset, and updates it if necessary.
     *
     * It calls @_searchData with its params. It returns
     * matched data as an argument to the 'dbsearch_results'
     * event called.
     *
     * It the query length is bigger than @config.minCharsForRemoteSearch
     * it queries remotely via an XHR.
     * The XHR is made after a timeout.
     * Once the remote data are fetched, the get appended to @_data and
     * @_searchData is called again the event 'dbsearch_results'
     * is triggered with the updated results.
     *
     * @param  {String|Number} query
     * @param  {Object}        options
     * @return {null or Array}
     */

    StorageFreak.prototype.search = function(query, options) {
      var results;
      if (options == null) {
        options = {};
      }
      results = this._searchData(query, options);
      this._trigger('dbsearch_results', results, query);
      this._resetRemote();
      if (query.length >= this.config.minCharsForRemoteSearch) {
        this._trigger('dbremote_search_in', this.config.XHRTimeout);
        return this.timeout = setTimeout(((function(_this) {
          return function() {
            return _this._getRemoteData(query, options).then(function() {
              results = _this._searchData(query, options);
              return _this._trigger('dbsearch_results', results, query, 'xhr');
            });
          };
        })(this)), this.config.XHRTimeout);
      }
    };

    StorageFreak.prototype.cleanup = function() {
      return this._resetRemote();
    };

    StorageFreak.prototype._resetRemote = function() {
      var _base;
      this._trigger('dbremote_search_reset');
      this.timeout && clearTimeout(this.timeout);
      return this.XHR_dfd && (typeof (_base = this.XHR_dfd).abort === "function" ? _base.abort() : void 0);
    };

    StorageFreak.prototype._searchData = function(query, options) {
      var id, match_func, name, re, results, search_type, sort_func, _ref;
      results = [];
      match_func = options.match_func || this.config.match_func;
      sort_func = options.sort_func || this.config.sort_func;
      search_type = options.search_type || this.config.search_type;
      query = this._createAccentIndependentQuery(query);
      re = this._createQueryRE(query, this.config.search_type);
      _ref = this._data;
      for (name in _ref) {
        id = _ref[name];
        if (match_func(re, name)) {
          results.push({
            id: id,
            name: name
          });
        }
      }
      this._resetSortingREs();
      results = results.sort((function(a, b) {
        return sort_func(query, a, b);
      }));
      return results.slice(0, this.config.maxResultsNum);
    };

    StorageFreak.prototype._getRemoteData = function(query, options) {
      var dfd, params, query_param;
      this._trigger("" + (options.event_name || 'dbremote_search') + "_start");
      dfd = new $.Deferred();
      params = {};
      if (query) {
        query_param = options.query || this.config.query;
        params[query_param] = query;
      }
      this.XHR_dfd = $.getJSON(options.url || this.config.url, params);
      this.XHR_dfd.then((function(_this) {
        return function(json_data) {
          return _this._getRemoteSuccess(json_data, dfd, options);
        };
      })(this), (function(_this) {
        return function(xhr) {
          return _this._getRemoteError(xhr, dfd, options);
        };
      })(this));
      return dfd;
    };

    StorageFreak.prototype._getRemoteSuccess = function(json_data, dfd, options) {
      this._data = $.extend({}, this._data, this._parseResponseData(json_data));
      this._updateDB(this._data);
      this._trigger("" + (options.event_name || 'dbremote_search') + "_end");
      return dfd.resolve();
    };

    StorageFreak.prototype._getRemoteError = function(xhr, dfd, options) {
      this._trigger("" + (options.event_name || 'dbremote_search') + "_error", xhr);
      return dfd.reject(xhr);
    };

    StorageFreak.prototype._getStorage = function() {
      if (localStorage) {
        return localStorage;
      } else {
        return new LocalStorageShim();
      }
    };

    StorageFreak.prototype._trigger = function() {
      var context, data, entry, event_name, _i, _len, _ref, _results;
      event_name = arguments[0], data = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      if (!this._events[event_name]) {
        return;
      }
      _ref = this._events[event_name];
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        entry = _ref[_i];
        context = entry.context || null;
        _results.push(entry.callback.apply(context, data));
      }
      return _results;
    };

    StorageFreak.prototype._isInvalidData = function() {
      var db_ts, ts;
      ts = new Date().getTime();
      db_ts = this._get(this._timestamp_key);
      if (db_ts) {
        db_ts = parseInt(db_ts, 10);
      }
      return db_ts === false || (ts - db_ts) > this.config.localCacheTimeout;
    };

    StorageFreak.prototype._parseResponseData = function(json_data) {
      var item, result, _i, _len;
      result = {};
      for (_i = 0, _len = json_data.length; _i < _len; _i++) {
        item = json_data[_i];
        result[item.name] = item.id;
      }
      return result;
    };

    StorageFreak.prototype._updateDB = function(data) {
      this._set(this._data_key, data);
      this._set(this._timestamp_key, new Date().getTime());
      return this._trigger('dbupdated');
    };

    StorageFreak.prototype._createAccentIndependentQuery = function(query) {
      var re, value, _i, _len, _ref;
      _ref = this.config.list_of_replacable_chars;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        value = _ref[_i];
        re = new RegExp("" + value[0] + "|" + value[1], 'ig');
        query = query.replace(re, "(?:" + value[0] + "|" + value[1] + ")");
      }
      return query;
    };

    StorageFreak.prototype._createQueryRE = function(query, type) {
      if (type == null) {
        type = 'infix';
      }
      if (type === 'infix') {
        return new RegExp("" + query, 'ig');
      }
      if (type === 'prefix') {
        return new RegExp("^" + query, 'ig');
      }
      if (type === 'token') {
        return new RegExp("\\b" + query + "\\b", 'ig');
      }
      if (type === 'absolute') {
        return new RegExp("^" + query + "$", 'ig');
      }
    };

    StorageFreak.prototype._createSortingREs = function(query) {
      var re_abs, re_p, re_t;
      re_abs = this._createQueryRE(query, 'absolute');
      re_t = this._createQueryRE(query, 'token');
      re_p = this._createQueryRE(query, 'prefix');
      return [re_abs, re_t, re_p];
    };

    StorageFreak.prototype._resetSortingREs = function() {
      return this.sort_criteria_res = null;
    };

    StorageFreak.prototype._get = function(key) {
      var value;
      if (typeof key !== 'string' || key === '') {
        return false;
      }
      key = "" + this._db_prefix + "." + key;
      try {
        value = JSON.parse(this._storage.getItem(key));
      } catch (_error) {
        return false;
      }
      if (value === null || value === '') {
        return false;
      } else {
        return value;
      }
    };

    StorageFreak.prototype._set = function(key, value) {
      if (typeof key !== 'string' || key === '') {
        return false;
      }
      if (value === null || value === void 0) {
        return false;
      }
      key = "" + this._db_prefix + "." + key;
      try {
        this._storage.setItem(key, JSON.stringify(value));
      } catch (_error) {
        return false;
      }
      return true;
    };

    return StorageFreak;

  })();
  return StorageFreak;
});

==> selectorablium-2.1.1.tgz: package/compiled/storage_freak.js <==
var __slice = [].slice;

define(['jquery', 'shims/local_storage_shim'], function($, LocalStorageShim) {
  var StorageFreak;
  StorageFreak = (function() {
    var fuzzy_sort, lexicographical_sort, regexp_sort;

    regexp_sort = function(re, a, b) {
      return (function(re_match_a, re_match_b) {
        if (re_match_a && !re_match_b) {
          return -1;
        }
        if (re_match_b && !re_match_a) {
          return 1;
        }
      })(this.config.match_func(re, a.name), this.config.match_func(re, b.name));
    };

    lexicographical_sort = function(a, b) {
      if (a.name < b.name) {
        return -1;
      }
      if (b.name < a.name) {
        return 1;
      }
    };

    fuzzy_sort = function(query, a, b) {
      var re, regexp_result, _i, _len, _ref;
      this.sort_criteria_res || (this.sort_criteria_res = this._createSortingREs(query));
      _ref = this.sort_criteria_res;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        re = _ref[_i];
        regexp_result = regexp_sort.call(this, re, a, b);
        re.lastIndex = 0;
        if (regexp_result) {
          return regexp_result;
        }
      }
      return lexicographical_sort(a, b);
    };

    StorageFreak.prototype._defaults = {
      namespace: 'selectorablium',
      sort_func: (function(query, a, b) {
        return fuzzy_sort.call(this, query, a, b);
      }),
      match_func: (function(RE, name) {
        var result;
        result = RE.test(name);
        RE.lastIndex = 0;
        return result;
      }),
      search_type: 'infix'
    };

    StorageFreak.prototype._required = ['url', 'name', 'query', 'app_name', 'XHRTimeout', 'maxResultsNum', 'localCacheTimeout', 'minCharsForRemoteSearch', 'list_of_replacable_chars'];

    function StorageFreak(options) {
      var attr, k, v, _i, _len, _ref, _ref1;
      if (this instanceof StorageFreak === false) {
        return new StorageFreak(options);
      }
      this.options = options;
      this.config = $.extend({}, this._defaults, this.options);
      _ref = this.config;
      for (k in _ref) {
        v = _ref[k];
        if ($.type(v) === 'function') {
          this.config[k] = $.proxy(v, this);
        }
      }
      _ref1 = this._required;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        attr = _ref1[_i];
        if (!this.config[attr]) {
          throw new Error("'" + attr + "' option is required");
        }
      }
      this._db_prefix = "" + this.config.namespace + "." + this.config.app_name;
      this._data_key = "" + this.config.name + "_data";
      this._timestamp_key = "" + this.config.name + "_timestamp";
      this._last_modified_key = "" + this.config.name + "_last_modified";
      this._data = {};
      this._events = {};
      this.timeout = null;
      this.XHR_dfd = null;
      this._storage = this._getStorage();
    }

    StorageFreak.prototype.on = function(event_name, callback, context) {
      var _base;
      if (context == null) {
        context = null;
      }
      if ((_base = this._events)[event_name] == null) {
        _base[event_name] = [];
      }
      return this._events[event_name].push({
        callback: callback,
        context: context
      });
    };

    StorageFreak.prototype.init = function() {
      if (this.config.cache_revalidate || this._isInvalidData()) {
        return this._getRemoteData('', {
          event_name: 'dbcreate'
        });
      } else {
        this._data = this._get(this._data_key);
        return new $.Deferred().resolve();
      }
    };

    StorageFreak.prototype.add = function(value, text) {
      var new_data;
      if (this._data[text]) {
        return;
      }
      new_data = {};
      new_data[text] = value;
      this._data = $.extend({}, this._data, new_data);
      return this._updateDB(this._data);
    };

    StorageFreak.prototype.searchByValue = function(val) {
      var text, value, _ref;
      _ref = this._data;
      for (text in _ref) {
        value = _ref[text];
        if (value === val) {
          return text;
        }
      }
      return false;
    };


    /**
     * The main API method.
     *
     * It queries the dataset, and updates it if necessary.
     *
     * It calls @_searchData with its params. It returns
     * matched data as an argument to the 'dbsearch_results'
     * event called.
     *
     * It the query length is bigger than @config.minCharsForRemoteSearch
     * it queries remotely via an XHR.
     * The XHR is made after a timeout.
     * Once the remote data are fetched, the get appended to @_data and
     * @_searchData is called again the event 'dbsearch_results'
     * is triggered with the updated results.
     *
     * @param  {String|Number} query
     * @param  {Object}        options
     * @return {null or Array}
     */

    StorageFreak.prototype.search = function(query, options) {
      var results;
      if (options == null) {
        options = {};
      }
      results = this._searchData(query, options);
      this._trigger('dbsearch_results', results, query);
      this._resetRemote();
      if (query.length >= this.config.minCharsForRemoteSearch) {
        this._trigger('dbremote_search_in', this.config.XHRTimeout);
        return this.timeout = setTimeout(((function(_this) {
          return function() {
            return _this._getRemoteData(query, options).then(function() {
              results = _this._searchData(query, options);
              return _this._trigger('dbsearch_results', results, query, 'xhr');
            });
          };
        })(this)), this.config.XHRTimeout);
      }
    };

    StorageFreak.prototype.cleanup = function() {
      return this._resetRemote();
    };

    StorageFreak.prototype._resetRemote = function() {
      var _base;
      this._trigger('dbremote_search_reset');
      this.timeout && clearTimeout(this.timeout);
      return this.XHR_dfd && (typeof (_base = this.XHR_dfd).abort === "function" ? _base.abort() : void 0);
    };

    StorageFreak.prototype._searchData = function(query, options) {
      var id, match_func, name, re, results, search_type, sort_func, _ref;
      results = [];
      match_func = options.match_func || this.config.match_func;
      sort_func = options.sort_func || this.config.sort_func;
      search_type = options.search_type || this.config.search_type;
      query = this._createAccentIndependentQuery(query);
      re = this._createQueryRE(query, this.config.search_type);
      _ref = this._data;
      for (name in _ref) {
        id = _ref[name];
        if (match_func(re, name)) {
          results.push({
            id: id,
            name: name
          });
        }
      }
      this._resetSortingREs();
      results = results.sort((function(a, b) {
        return sort_func(query, a, b);
      }));
      return results.slice(0, this.config.maxResultsNum);
    };

    StorageFreak.prototype._getRemoteData = function(query, options) {
      var dfd, headers, last_modified, params, query_param;
      this._trigger("" + (options.event_name || 'dbremote_search') + "_start");
      dfd = new $.Deferred();
      params = {};
      if (query) {
        query_param = options.query || this.config.query;
        params[query_param] = query;
      }
      headers = {};
      last_modified = this._get(this._last_modified_key);
      if (!query && last_modified) {
        headers['If-Modified-Since'] = new Date(last_modified).toUTCString();
      }
      this.XHR_dfd = $.ajax({
        dataType: "json",
        url: options.url || this.config.url,
        headers: headers,
        data: params
      });
      this.XHR_dfd.then((function(_this) {
        return function(json_data, textStatus, xhr) {
          if (xhr.status === 304) {
            return _this._getRemoteNotModified(xhr, dfd, options);
          } else {
            return _this._getRemoteSuccess(json_data, xhr, dfd, options);
          }
        };
      })(this), (function(_this) {
        return function(xhr) {
          return _this._getRemoteError(xhr, dfd, options);
        };
      })(this));
      return dfd;
    };

    StorageFreak.prototype._getRemoteSuccess = function(json_data, xhr, dfd, options) {
      var last_modified, last_modified_ts;
      this._data = $.extend({}, this._data, this._parseResponseData(json_data));
      if (last_modified = xhr.getResponseHeader('Last-Modified')) {
        last_modified_ts = new Date(last_modified).getTime();
      }
      this._updateDB(this._data, last_modified_ts);
      this._trigger("" + (options.event_name || 'dbremote_search') + "_end");
      return dfd.resolve();
    };

    StorageFreak.prototype._getRemoteNotModified = function(xhr, dfd, options) {
      this._data = this._get(this._data_key);
      this._trigger("" + (options.event_name || 'dbremote_search') + "_end");
      return dfd.resolve();
    };

    StorageFreak.prototype._getRemoteError = function(xhr, dfd, options) {
      this._trigger("" + (options.event_name || 'dbremote_search') + "_error", xhr);
      return dfd.reject(xhr);
    };

    StorageFreak.prototype._getStorage = function() {
      if (localStorage) {
        return localStorage;
      } else {
        return new LocalStorageShim();
      }
    };

    StorageFreak.prototype._trigger = function() {
      var context, data, entry, event_name, _i, _len, _ref, _results;
      event_name = arguments[0], data = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      if (!this._events[event_name]) {
        return;
      }
      _ref = this._events[event_name];
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        entry = _ref[_i];
        context = entry.context || null;
        _results.push(entry.callback.apply(context, data));
      }
      return _results;
    };

    StorageFreak.prototype._isInvalidData = function() {
      var db_ts, ts;
      ts = new Date().getTime();
      db_ts = this._get(this._timestamp_key);
      if (db_ts) {
        db_ts = parseInt(db_ts, 10);
      }
      return db_ts === false || (ts - db_ts) > this.config.localCacheTimeout;
    };

    StorageFreak.prototype._parseResponseData = function(json_data) {
      var item, result, _i, _len;
      result = {};
      for (_i = 0, _len = json_data.length; _i < _len; _i++) {
        item = json_data[_i];
        result[item.name] = item.id;
      }
      return result;
    };

    StorageFreak.prototype._updateDB = function(data, last_modified_ts) {
      this._set(this._data_key, data);
      this._set(this._timestamp_key, new Date().getTime());
      if (last_modified_ts) {
        this._set(this._last_modified_key, last_modified_ts);
      }
      return this._trigger('dbupdated');
    };

    StorageFreak.prototype._createAccentIndependentQuery = function(query) {
      var re, value, _i, _len, _ref;
      _ref = this.config.list_of_replacable_chars;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        value = _ref[_i];
        re = new RegExp("" + value[0] + "|" + value[1], 'ig');
        query = query.replace(re, "(?:" + value[0] + "|" + value[1] + ")");
      }
      return query;
    };

    StorageFreak.prototype._createQueryRE = function(query, type) {
      if (type == null) {
        type = 'infix';
      }
      if (type === 'infix') {
        return new RegExp("" + query, 'ig');
      }
      if (type === 'prefix') {
        return new RegExp("^" + query, 'ig');
      }
      if (type === 'token') {
        return new RegExp("\\b" + query + "\\b", 'ig');
      }
      if (type === 'absolute') {
        return new RegExp("^" + query + "$", 'ig');
      }
    };

    StorageFreak.prototype._createSortingREs = function(query) {
      var re_abs, re_p, re_t;
      re_abs = this._createQueryRE(query, 'absolute');
      re_t = this._createQueryRE(query, 'token');
      re_p = this._createQueryRE(query, 'prefix');
      return [re_abs, re_t, re_p];
    };

    StorageFreak.prototype._resetSortingREs = function() {
      return this.sort_criteria_res = null;
    };

    StorageFreak.prototype._get = function(key) {
      var value;
      if (typeof key !== 'string' || key === '') {
        return false;
      }
      key = "" + this._db_prefix + "." + key;
      try {
        value = JSON.parse(this._storage.getItem(key));
      } catch (_error) {
        return false;
      }
      if (value === null || value === '') {
        return false;
      } else {
        return value;
      }
    };

    StorageFreak.prototype._set = function(key, value) {
      if (typeof key !== 'string' || key === '') {
        return false;
      }
      if (value === null || value === void 0) {
        return false;
      }
      key = "" + this._db_prefix + "." + key;
      try {
        this._storage.setItem(key, JSON.stringify(value));
      } catch (_error) {
        return false;
      }
      return true;
    };

    return StorageFreak;

  })();
  return StorageFreak;
});
